/**
 * \file shape_generation.cpp
 * \brief ****
 * \author Luis Alvarez \n \n
*/


#include <iostream>
#include <vector>
#include <math.h>
#include "shape_generation.h"



#define ami_PI 3.14159265358979323846
#define CERO 1e-12

vector<float> random_ordered_vector(int VectorSize,float min_extrema,float max_extrema){
 if(VectorSize<=0) return vector<float>();
 vector<float> v(VectorSize);
 v[0]=min_extrema+(max_extrema-min_extrema)*rand()/RAND_MAX;
 float min_distance=(max_extrema-min_extrema)/VectorSize/4.;
 for(int k=1;k<v.size();k++){
   float distance=0.;
   while(distance<min_distance){
     distance=min_distance+1;
     v[k]=min_extrema+(max_extrema-min_extrema)*rand()/RAND_MAX;
     for(int n=0;n<k;n++){
       float temp=fabs(v[k]-v[n]);
       if(temp<distance) distance=temp;
     }
   }
 }
 // WE ORDER THE VECTOR
 for(int k=0;k<v.size();k++){
    for(int n=k+1;n<v.size();n++){
      if(v[k]<v[n]){
        float temp=v[k]; v[k]=v[n]; v[n]=temp;
      }
    }
 }
 return v;
}
/**********************************************************
SHAPE GENERATOR
**********************************************************/
shape shape_generator::new_shape(int &shape_database,int shape_index){
  triangulation T;
  if(shape_index<0){
    shape_index=rand();
  }
  shape_database=shape_database%6;
  switch(shape_database) {
    case 0 : /// basic shapes
      switch(shape_index%7){
        case 0 : /// triangle
          //T=triangulation();
          //printf("case 0-0-0 : triangle\n");
          T.p.push_back(point2d(1.,0.));
          T.p.push_back(point2d(cos(2.*ami_PI/3.),sin(2.*ami_PI/3.)));
          T.p.push_back(point2d(cos(4.*ami_PI/3.),sin(4.*ami_PI/3.)));
          T.fr.push_back(1); T.fr.push_back(1); T.fr.push_back(1);
          T.t.push_back(triangle(0,1,2));
          T.n.push_back(triangle(-1,-1,-1));
          return shape(T);
        case 1 : /// square
          //printf("case 0-0-1 : square\n");
          return shape(triangulation(point2d(-1.,-1.),point2d(1.,1.)));
        case 2 : /// circle
          //printf("case 0-0-2 : circle\n");
          return shape(triangulation(point2d(0.,0.),1.,60));
        case 3 : /// hexagon
          //printf("case 0-0-3 : hexagon\n");
          return shape(triangulation(point2d(0.,0.),1.,6));
        case 4 : /// star
          //printf("case 0-0-4 : star\n");
          {
            vector<point2d> pV;
            for(int k=0;k<10;k++){
            float a=2.*k*ami_PI/10.;
            if(k%2==0) pV.push_back(point2d(cos(-a),sin(-a)));
            else pV.push_back(point2d(cos(-a)*0.4,sin(-a)*0.4));
            //printf("p[%d]=(%lf,%lf) norm=%lf\n",k,pV[k].x,pV[k].y,pV[k].norm());
            }
            //system("pause");
            return shape(triangulation(pV));
          }
        case 5 : /// polygon of Npoints
          //printf("case 0-0-5 : polygon\n");
          {
            int N=4+7*rand()/RAND_MAX;
            vector<point2d> pV(N);
            vector<float> a=random_ordered_vector(N,0.,2.*ami_PI);
            for(int k=0;k<N;k++){
              pV[k]=(point2d(cos(a[k]),sin(a[k])));
            }
            return shape(triangulation(pV));
          }
        case 6 : /// rectangle band with a random strip thick
          //printf("case 0-0-6 : rectangle band\n");
          {
            float strip_thick=0.3+0.4*rand()/RAND_MAX;
            return shape(triangulation(strip_thick,point2d(-1.,-1.),point2d(1.,1.)));
          }

        case 7 :
          break;
        case 8 :
          break;
        case 9 :
          break;
        case 10 :
          break;
        case 11 :
          break;
      }
      break;
    case 1 : /// constructor based on segment random evolution
      switch(shape_index%3){
        case 0 :
          {
            //printf("case 0-1-0 : brownian caotic evolution with variable segment thick \n");
            float radius=1.+1.*rand()/RAND_MAX;
            triangulation T0(point2d(-radius,-radius),point2d(radius,radius));
            float r=0.1+0.2*rand()/RAND_MAX;
            float a=-0.01+0.02*rand()/RAND_MAX;
            float da=0.5+1.*rand()/RAND_MAX;
            float dt=0.1,dr=0.2*rand()/RAND_MAX;
            triangulation T1(T0,point2d(0.,0.),dt,a,-da+a,da+a,r,-dr,dr,200);
            T1.put_horizontal();
            T1=T1-T1.p[T1.p.size()/2];
            T1.rebuild(dt);
            T1.rebuildOnlyContour();
            return shape(T1);
          }
        case 1 :
          {
            //printf("case 0-1-1 : brownian smooth evolution with fix segment thick\n");
            float radius=0.5+3.5*rand()/RAND_MAX;
            triangulation T0(point2d(-radius,-radius),point2d(radius,radius));
            float r=0.025,da=0.2,a=0.,dt=0.1,dr=0.;
            triangulation T1(T0,point2d(0.,0.),dt,a,-da+a,da+a,r,-dr,dr,5000);
            T1.put_horizontal();
            T1=T1-T1.p[T1.p.size()/2];
            T1.rebuild();
            return shape(T1);
          }
        case 2 :
          {
            //printf("case 0-1-2: brownian caotic evolution with fix segment thick\n");
            float radius=0.5+3.5*rand()/RAND_MAX;
            triangulation T0(point2d(-radius,-radius),point2d(radius,radius));
            float r=0.025,da=1.,a=0.,dt=0.1,dr=0.;
            triangulation T1(T0,point2d(0.,0.),dt,a,-da+a,da+a,r,-dr,dr,200);
            T1.put_horizontal();
            T1=T1-T1.p[T1.p.size()/2];
            T1.rebuild();
            return shape(T1);
          }

      }
      break;
    case 2 : /// Kandisky circle
      //printf("case 0-2 : Kandisky circle\n");
      Kc=kandisky_circle(shape_index%1);
      return (Kc.new_shape()).normalize();
    case 3 : /// Kandisky rectangle
      //printf("case 0-3 : Kandisky rectangle\n");
      Kr=kandisky_rectangle(shape_index%1);
      return (Kr.new_shape()).normalize();
    case 4 : /// tree
      //printf("case 0-4 : Kandisky tree \n");
      {
        shape S=shape_generator().new_shape(301);
        float zoom=3.+3.*rand()/RAND_MAX;
        contouring(S,S,-0.1,100,10,0.001,zoom);
        zoom=0.75+0.5*rand()/RAND_MAX;
        contouring(S,S,0.4,300,30,0.01,zoom);
//        S.print();
//        printf("S.shapeChild.size()=%d\n",S.shapeChild.size());
//        system("pause");
        for(int k=0;k<S.shapeChild.size();k++){
          for(int n=0;n<S.shapeChild[k].T.p.size();n++){
            if(S.shapeChild[k].T.p[n].y-0.1<S.T.p[0].y){
              S.shapeChild.erase(S.shapeChild.begin()+k);
              k--;
              break;
            }
          }
        }
        return (S.normalize());
      }
    case 5 :
    {
      //printf("case 0-5 : rosette  \n");
      Kr=kandisky_rectangle(shape_index%1);
      shape S2=Kr.new_shape();
      S2.symmetry(point2d(0.,0.),16);
      return S2.normalize();
    }
    case 6 :
      switch(shape_index%6){
        case 0 :
        case 1 :
         break;
      }
      break;
    case 7 :
      switch(shape_index%6){
        case 0 :
        case 1 :
          break;
      }
      break;
    case 8 :
      switch(shape_index%6){
        case 0 :
        case 1 :
           break;
      }
      break;
    case 9 :
      switch(shape_index%6){
        case 0 :
        case 1 :
            break;
      }
      break;
  }
};


shape shape_generator::new_shape(int shape_type_new){

  /// TRIANGLE
  if(shape_type_new==0){
    triangulation T;
    T.p.push_back(point2d(1.,0.));
    T.p.push_back(point2d(cos(2.*ami_PI/3.),sin(2.*ami_PI/3.)));
    T.p.push_back(point2d(cos(4.*ami_PI/3.),sin(4.*ami_PI/3.)));
    T.fr.push_back(1); T.fr.push_back(1); T.fr.push_back(1);
    T.t.push_back(triangle(0,1,2));
    T.n.push_back(triangle(-1,-1,-1));
    return shape(T);
  }
  /// SQUARE
  else if(shape_type_new==1){
    return shape(triangulation(point2d(-1.,-1.),point2d(1.,1.)));
  }
  /// CIRCLE
  else if(shape_type_new==2){
    return shape(triangulation(point2d(0.,0.),1.,20));
  }

 /// KANDISKY CIRCLE : shape_type_new in [100,200)
 if(99<shape_type_new && shape_type_new<200){
    shape_type_new-=100;
    if(shape_type_new==0) return Kc.new_shape();
    Kc=kandisky_circle(shape_type_new-1);
    return Kc.new_shape();
 }

 /// KANDISKY RECTANGLE shape_type_new in [200,300)
 else if (199<shape_type_new && shape_type_new<300){
    shape_type_new-=200;
    if(shape_type_new==0) return Kr.new_shape();
    Kr=kandisky_rectangle(shape_type_new-1);
    return Kr.new_shape();
 }
 /// LEAF shape_type_new in [300,400)
  else if (299<shape_type_new && shape_type_new<400){
    shape_type_new-=300;
    if(shape_type_new==0) return Leaf.new_shape();
    Leaf=leaf(shape_type_new-1);
    return Leaf.new_shape();
 }

 return shape();

}

/// OBSOLETO
shape shape_generator::new_shape(int shape_type_new,triangulation &T1,int index_triangle,int index_segment,float min_distance,float zoom){

 shape S;
 /// KANDISKY CIRCLE : shape_type_new in [100,200)
 if(99<shape_type_new && shape_type_new<200){
    shape_type_new-=100;
    //if(shape_type_new==0) return Kc.new_shape();
    Kc=kandisky_circle(shape_type_new-1);
    S=Kc.new_shape();
    //printf("S.T.p.size()=%d\n",S.T.p.size()); system("pause");
 }

 /// KANDISKY RECTANGLE shape_type_new in [200,300)
 else if (199<shape_type_new && shape_type_new<300){
    shape_type_new-=200;
    //if(shape_type_new==0) return Kr.new_shape();
    Kr=kandisky_rectangle(shape_type_new-1);
    S=Kr.new_shape();
 }
 /// LEAF shape_type_new in [300,400)
  else if (299<shape_type_new && shape_type_new<400){
    shape_type_new-=300;
    //if(shape_type_new==0) return Leaf.new_shape();
    Leaf=leaf(shape_type_new,T1,index_triangle,index_segment);
    //printf("Leaf.dt=%lf Leaf.Niter=%d\n",Leaf.dt,Leaf.Niter); system("pause");
    S=Leaf.new_shape();
 }

 if(S.T.p.size()==0) return shape();
 if(zoom!=1.) S=S*zoom;
 affinity A=connectT12T0(T1,index_triangle,index_segment,S.T,1,2,min_distance,1);
 return S*A;

}



//// GENERATOR OF KANDISKY CIRCLE
shape kandisky_circle_generator(
float radius /** radius of kandisky circle */,
int Nlevels /** number of inside circles */,
int CircleNpoints /** number of points to discretize the circle*/)
{
  point2d center(0.,0.);
  float angle0=2.*M_PI*((float) rand()/RAND_MAX-0.5);
  float par0=0.2*((float) rand()/RAND_MAX);
  float angle1=2.*M_PI*((float) rand()/RAND_MAX-0.5);
  float par1=0.2+0.2*((float) rand()/RAND_MAX);
  triangulation T(center,radius,CircleNpoints);
  //printf("T.p.size()=%d CircleNpoints=%d \n",T.p.size(),CircleNpoints);

  shape S(1,T.deformation(center,angle0,par0,angle1,par1));
  shape *Sp;
  Sp=&S;
  for(int l=1;l<Nlevels;l++){
    triangulation T2(center,radius-radius*l/(Nlevels+1.),CircleNpoints);
    bool inclusion=false;
    while(inclusion==false){
      angle0=2.*M_PI*((float) rand()/RAND_MAX-0.5);
      par0=0.2*((float) rand()/RAND_MAX);
      angle1=2.*M_PI*((float) rand()/RAND_MAX-0.5);
      par1=0.2+0.2*((float) rand()/RAND_MAX);
      triangulation T3=T2.deformation(center,angle0,par0,angle1,par1); //deformed circle
      //triangulation T3=T2;
      inclusion=(*Sp).inclusion_test(T3); //we check if each circle is included in its father circle
      if(inclusion==true){
        (*Sp).shapeChild.push_back(shape((*Sp).tree_level+1,T3)); // we add the new circle as a child shape
        Sp=&((*Sp).shapeChild[0]); // we update the shape pointer
      }
    }
  }
  //S.extrema_update();

  return S;
}

// GENERATOR OF KANDISKY RECTANGLE
shape kandisky_rectangle_generator(
float width /** rectangle width */,
float height /** rectangle height */,
int Nlevels /** number of inside rectangles */,
float min_distance /** min distance between points of the rectangle */)
{
  point2d center(0.,0.);
  float angle0=2.*M_PI*((float) rand()/RAND_MAX-0.5);
  float par0=0.2*((float) rand()/RAND_MAX);
  float angle1=2.*M_PI*((float) rand()/RAND_MAX-0.5);
  float par1=0.2+0.2*((float) rand()/RAND_MAX);
  //triangulation T(center,radius,CircleNpoints);
  triangulation T(point2d(-width/2,-height/2),point2d(width/2,height/2),min_distance);

  shape S(1,T.deformation2(center,angle0,par0,angle1,par1));
  shape *Sp;
  Sp=&S;
  for(int l=1;l<Nlevels;l++){
    //triangulation T2(center,radius-radius*l/(Nlevels+1.),CircleNpoints);
    float width2=width-width*l/(Nlevels+1.);
    float height2=height-height*l/(Nlevels+1.);
    triangulation T2(point2d(-width2/2,-height2/2),point2d(width2/2,height2/2),min_distance);
    bool inclusion=false;
    while(inclusion==false){
      angle0=2.*M_PI*((float) rand()/RAND_MAX-0.5);
      par0=0.2*((float) rand()/RAND_MAX);
      angle1=2.*M_PI*((float) rand()/RAND_MAX-0.5);
      par1=0.2+0.2*((float) rand()/RAND_MAX);
      triangulation T3=T2.deformation2(center,angle0,par0,angle1,par1); //deformed circle
      inclusion=(*Sp).inclusion_test(T3); //we check if each circle is included in its father circle
      if(inclusion==true){
        (*Sp).shapeChild.push_back(shape((*Sp).tree_level+1,T3)); // we add the new circle as a child shape
        Sp=&((*Sp).shapeChild[0]); // we update the shape pointer
      }
    }
  }
  return S;
}


// GENERATION OF A KANDISKY RECTANGLE FRAME
shape kandisky_rectangle_frame_generator(
point2d  pmin /** frame window min point coordinates */,
point2d pmax /** frame window max point coordinates */,
int NCircles_x /** number of  horizontal kandisky rectangles taking into account the corners */,
int NCircles_y /** number of vertical kandisky rectangles without taking into account the corners */,
int Nlevels /** number of levels (inside rectangles) of the kandisky rectangle */,
float min_distance /** min distance between rectangle points */)
{
  if(NCircles_x==0 && NCircles_y==0) return shape();
  float hx,hy;
  if(NCircles_x!=0) hx=(pmax.x-pmin.x)/NCircles_x;
  else hx=pmax.x-pmin.x;
  if(NCircles_y!=0) hy=(pmax.y-pmin.y-2*hx)/NCircles_y;
  else hy=pmax.y-pmin.y;
  shape S;
  triangulation T0(pmin,pmax);
  triangulation T1(pmin+point2d(hx,hy),pmax-point2d(hx,hy));
  if(NCircles_x!=0 && NCircles_y!=0) S=shape(1,T0-T1);
  else S=shape(1,T0);
  float width=0.8*hx;
  float height=0.8*hy;
  for(int k=0;k<NCircles_x;k++){
    point2d d0=pmin+point2d(hx*0.5+k*hx,hy*0.5);
    point2d d1=point2d(pmin.x+hx*0.5+k*hx,pmax.y-hy*0.5);
    bool inclusion=false;
    while(inclusion==false){
      //shape S0=kandisky_circle_generator(2.*hx/5.,Nlevels,CircleNpoints)+d0;
      shape S0=kandisky_rectangle_generator(width,height,Nlevels,min_distance)+d0;
      if(S.inclusion_test(S0)==true){
        S.shapeChild.push_back(S0);
        inclusion=true;
      }
    }
    if(NCircles_y>0){
      inclusion=false;
      while(inclusion==false){
        shape S0=kandisky_rectangle_generator(width,height,Nlevels,min_distance)+d1;
        if(S.inclusion_test(S0)==true){
          S.shapeChild.push_back(S0);
          inclusion=true;
        }
      }
    }
  }
  for(int k=0;k<NCircles_y;k++){
    point2d d0=pmin+point2d(hx*0.5,hx+hy*0.5+k*hy);
    point2d d1=point2d(pmax.x-hx*0.5,hx+pmin.y+hy*0.5+k*hy);
    bool inclusion=false;
    while(inclusion==false){
      shape S0=kandisky_rectangle_generator(width,height,Nlevels,min_distance)+d0;
      if(S.inclusion_test(S0)==true){
        S.shapeChild.push_back(S0);
        inclusion=true;
      }
    }
    if(NCircles_x>0){
      inclusion=false;
      while(inclusion==false){
        shape S0=kandisky_rectangle_generator(width,height,Nlevels,min_distance)+d1;
        if(S.inclusion_test(S0)==true){
          S.shapeChild.push_back(S0);
          inclusion=true;
        }
      }
    }
  }

  return S;

}

triangulation leaf::T()
{
  triangulation T;
  vector<point2d> pV;
  vector<float> rV;
  pV.push_back(p0);
  rV.push_back(r0);
  float dan=(a1-a0)/Niter;
  float drn=(r1-r0)/Niter;
  float a=a0;
  float r=r0;
  point2d v_=point2d(sin(a0),cos(a0));
  for(int k=0;k<Niter;k++){
    if(r<0.01) r=0.01;
     p0=p0+v_*dt;
     pV.push_back(p0);
     rV.push_back(r);
     //printf("p0=(%lf,%lf)\n",p0.x,p0.y);
     float w=(float) rand()/RAND_MAX;
     float dr=w*drl+(1-w)*drr+drn;
     r+=dr;
     w=(float) rand()/RAND_MAX;
     float da=w*dal+(1-w)*dar+dan;
     a+=da;
     if(a0==0 && a1==0){
       float co,si=sin(a);
       bool change=false;
       if((p0.x<0 && si<0.) || (p0.x>0 && si>0.)){
        co=cos(a);
        si=si-p0.x;
        a=atan2(si,co);
       }
     }
     v_=point2d(sin(a),cos(a));
  }
 // triangulation T1(T0,p0,dt,0,-dangle+angle0,dangle+angle0,0.05*scale,-0.003*scale,0.0*scale,Niter);
 return triangulation(T0,pV,rV);

//  // WE ADD A CIRCLE AT THE END
//  int Nf=T1.p.size()-1;
//  //printf("Nf=%d\n",Nf);
//  if (Nf<2) return triangulation();
//  point2d p2=(T1.p[Nf]+T1.p[Nf-1])*0.5;
//  int Np=12; //2+(100.*dt);
//  float radius=(T1.p[Nf]-T1.p[Nf-1]).norm()*0.5;
//  triangulation T2=T1+triangulation(p2,radius,Np);
  //triangulation T2=T1+triangulation(T1.p[Nf-2],0.02, Np )+triangulation(T1.p[Nf-1],0.03, Np )+triangulation(T1.p[Nf],0.04, Np ) ;

//  T2.clean();
//  T2.update_n();
//  T2.rebuildOnlyContour(1e-10);

//  return T2;

}

//leaf(){p0.x=p0.y=0.; dt=0.05; a0=a1=0.; dal=-0.1; dar=0.1; r=0.05; drl=-0.01; drr=0.01; min_r=0.01; Niter=50; T=build_triangulation();}

leaf::leaf(int type){
  switch(type) {
    case 0 :
    {
      p0.x=p0.y=0.; dt=0.05; a0=a1=0.; dal=-0.1; dar=0.1;  drl=-0.0015; drr=0.; min_r=0.01; Niter=25;
      r0=r1=0.05+0.05*rand()/RAND_MAX;
      break;
    }
    case 1 :
    {
      p0.x=p0.y=0.; dt=0.05; a0=a1=0.; dal=-0.1; dar=0.1; r0=r1=0.05; drl=-0.0075; drr=0.; min_r=0.01; Niter=10;
      break;
    }
    case 2 :
    {
      p0.x=p0.y=0.; dt=0.05; a0=a1=0.; dal=-0.1; dar=0.1; r0=r1=0.05; drl=-0.0075; drr=0.; min_r=0.01; Niter=10;
      break;
    }
    default :
    {
      leaf();
      break;
    }
  }
}

leaf::leaf(int type,triangulation &T1,int indexTriangle,int indexSegment){
    //printf("indexTriangle=%d,indexSegment=%d\n",indexTriangle,indexSegment);
    p0.x=p0.y=0.; dt=0.05;  dal=-0.1; dar=0.1; drl=-0.0015; drr=0.0015; min_r=0.005; Niter=10;
    float max_segment_length=T1.max_segment_length(indexTriangle);
    point2d p=T1.p[T1.t[indexTriangle].v[(indexSegment+1)%3]]-T1.p[T1.t[indexTriangle].v[(indexSegment+0)%3]];
    a0=atan2(-p.y,-p.x);
    r1=min_r;
    if(type==0){ a1=a0/2.; r0=max_segment_length/2.; Niter=1+180*r0;}
    else if(type==1){ a1=0.; r0=max_segment_length/2.; Niter=1+180*r0;}
    else if(type==2){ a1=a0/2.; r0=max_segment_length/4.; Niter=1+180*r0;}
    else if(type==3){ a1=0.; r0=max_segment_length/4.; Niter=2+360*r0;}
    else if(type==4){ a1=a0/2.; r0=max_segment_length/2.; Niter=2+360*r0;}
    else if(type==5){ a1=0.; r0=max_segment_length/2.; Niter=2+360*r0;}
    else if(type==6){ a1=a0/2.; r0=max_segment_length/4.; Niter=2+360*r0;}
    else if(type==7){ a1=0.; r0=max_segment_length/4.; Niter=2+360*r0;}

}

/// CONTOURING A SHAPE WITH ANOTHER SHAPE TYPE
void contouring(shape &S0,shape &S,float min_distance,int shape_type,int Ntries,float distance,float zoom){
  vector<int> SelectedTriangles, SelectedSegments;
  vector<int> SelectedPoints;
  if(min_distance>0.) S.T.boundary_segments(min_distance,SelectedTriangles,SelectedSegments,SelectedPoints);
  else{
    if(S.T.t.size()>1){
      SelectedTriangles.push_back(S.T.t.size()-2);
      SelectedSegments.push_back(2);
    }
  }
  for(int m=0;m<SelectedTriangles.size();m++){
    int Nmax=0;
    float scale=exp(-log(2.)/Ntries),zoom2=zoom;
    while(Nmax++<Ntries){
      shape S1=shape_generator().new_shape(shape_type,S.T,SelectedTriangles[m],SelectedSegments[m],distance,zoom2);
      if(S1.T.p.size()>0 ){
        if(S0.intersection_test(S1)==false){
          //printf("1\n");
          S.shapeChild.push_back(S1);
          S0.extrema_update();
          break;
        }
      }
      zoom2*=scale;
    }
  }
}

/// ADD TO A COLLECTION OF SHAPES TRIANGULATIONS STORED IN A REPOSITORY
void ReadShapeDataBase(vector <shape> &SDB,char RepositoryName[]){
  char name[400];
  //sprintf(name,"dir \"%s\" /TW /ON /B > TriangulationList.txt",RepositoryName);
  //printf("%s\n",name);
  //system(name);
  //system("pause");
  FILE *f;
  //f = fopen("TriangulationList.txt", "r");
  sprintf(name,"%sTriangulationList.txt",RepositoryName);
  printf("name=%s\n",name);
  f = fopen(name, "r");
  if (f == NULL){
    printf("\nNo se pueden leer los ficheros de triangulaciones\n");
    //system("pause");
    { int ii_; scanf("%d",&ii_);}
    return ;
  }
  while(feof(f)==0){
    char nameT[200];
    fscanf(f,"%s\n",nameT);
    sprintf(name,"%s%s",RepositoryName,nameT);
    printf("%s   ",name);
    triangulation T(name);
    T.normalize();
    T.fix_triangle_orientation();
    T.update_n();
    T.rebuild();
    T.rebuild();
    T.rebuild();
    T.rebuild();

    shape S(0,T);
    S.extrema_update();
    S.print();
    SDB.push_back(S);
  }

}

/// ADD TO A COLLECTION OF PALETTES GIVEN BY BMP IMAGES STORED IN A REPOSITORY
void ReadPaletteDataBase(vector <imageBMP> &PDB,char RepositoryName[]){
  char name[300];
  //sprintf(name,"dir \"%s\" /TW /ON /B  > PaletteList.txt",RepositoryName);
  //printf("%s\n",name);
  //system(name);
  FILE *f;
  sprintf(name,"%s/PaletteList.txt",RepositoryName);
  printf("PaletteName=%s\n",name);
  //f = fopen("PaletteList.txt", "r");
  f = fopen(name, "r");
  if (f == NULL){
    printf("\nNo se pueden leer los ficheros de Paletas\n");
    //system("pause");
    { int ii_; scanf("%d",&ii_);}
    return ;
  }
  while(feof(f)==0){
    char nameT[200];
    fscanf(f,"%s\n",nameT);
    sprintf(name,"%s%s",RepositoryName,nameT);
    printf("%s\n",name);

    PDB.push_back(imageBMP(name));

//    imageBMP sortHSV=PDB[PDB.size()-1].sortHSV();
//    sortHSV.write(nameT);
  }

}
